use serde::{Deserialize, Serialize};

#[derive(Clone, Copy, Serialize, Deserialize, Debug)]
pub struct LedStripConfigOfBorders {
    pub top: Option<LedStripConfig>,
    pub bottom: Option<LedStripConfig>,
    pub left: Option<LedStripConfig>,
    pub right: Option<LedStripConfig>,
}

#[derive(Clone, Copy, Serialize, Deserialize, Debug)]
pub struct LedStripConfig {
    pub index: usize,
    pub global_start_position: usize,
    pub global_end_position: usize,
}

#[derive(Clone, Copy, Serialize, Deserialize, Debug)]
pub struct DisplayConfig {
    pub id: u32,
    pub index_of_display: usize,
    pub display_width: usize,
    pub display_height: usize,
    pub led_strip_of_borders: LedStripConfigOfBorders,
    pub scale_factor: f32,
}

impl LedStripConfigOfBorders {
    pub fn default() -> Self {
        Self {
            top: None,
            bottom: None,
            left: None,
            right: None,
        }
    }
}

impl DisplayConfig {
    pub fn default(
        id: u32,
        index_of_display: usize,
        display_width: usize,
        display_height: usize,
        scale_factor: f32,
    ) -> Self {
        Self {
            id,
            index_of_display,
            display_width,
            display_height,
            led_strip_of_borders: LedStripConfigOfBorders::default(),
            scale_factor,
        }
    }
}
